/*
  File: Lookit.m
  Authors: Eric Bergerson & Matt Jacobus
  Modification History:

  Created 10/12/90

  Copyright (c) 1990 Objective Technologies, Inc.
  All Rights Reserved.
*/

#import <appkit/appkit.h>
#import <otikit/graphkit.h>
#import "Lookit.h"

static NXRect nullFrame = {{0.0, 0.0}, {0.0, 0.0}};

@implementation Lookit

+ new
{
  int i;

  self = [super new];
  path = [[OTGString alloc] init];
  globalDataMngr = [[OTDataMngr alloc] init];
  oTGraph = nil;
  oTDataMngr = nil;

  cells[0] = [[OTLineCell alloc] init];
  cells[1] = [[OTHiLoCell alloc] init];
  cells[2] = [[OTBarCell alloc] init];
  cells[3] = [[OTMarkCell alloc] init];
  cells[4] = [[OTScatterCell alloc] init];
  cells[5] = [[OTGridCell alloc] init];

  for(i=0; i<CELLTYPES; i++){
    [cells[i] setContinuous: YES];
  }

  myCursor = nil;
  myPoint = nil;
  crossLives = NO;
  lineOn = NO;
  globalDataMngr = nil;
  lit = NO;
  graphEnabled = YES;

  return self;
}

- free
{
  int i;

  [path free];
  for(i=0; i<CELLTYPES; i++)
    [cells[i] free];

  return [super free];
}

- setOTDataMngr: anObject
{
  oTDataMngr = anObject;

  return self;
}

- setOTGraph: anObject
{
  oTGraph = anObject;
  [[oTGraph oTGraphBody] setAxisMajorWidth: 0.05];
  [[[oTGraph oTGraphBody] cellAt:G_GRAPH] setContinuous:YES];
  [oTGraph setAutodisplay: NO];
  return self;
}

- setOTLegend:anObject
{

  oTLegend = anObject;
  [oTLegend setAutodisplay: NO];

  return self;
}

- setScrollView: anObject
{
  scrollView = anObject;
  return self;
}

- setAxisValueField: anObject
{
  axisValueField = anObject;
  return self;
}

- setAxisMatrix: anObject
{
  axisMatrix = anObject;
  return self;
}

- setTitleF: anObject
{
  titleF = anObject;
  return self;
}

- Open:sender
{
  const char *const openTypes[2] = {"dm",NULL};
  id openPanel;

  openPanel = [OpenPanel new];
  if([openPanel runModalForTypes:openTypes] &&
     [openPanel filename]){
    [path strcpy: [openPanel filename]];
    if(![self readFromFile: self]){
      return nil;
    }
  }

  return self;
}

- readFromFile:sender
{
  id aDataMngr;
  id scale, subscales, labeler;
  int len,i;

  if (oTDataMngr) {
    scale = [oTGraph oTScale];
    [scale freeAllSubdata];
    [oTDataMngr free];
    if ([scale hasSubscales]) {
      subscales = [scale subscales];
      len = [subscales count];
      for(i=0;i<len;i++)
	[[subscales removeObjectAt: 0] free];
      /* Set the left and right axis (and labelers) to use main 
	 scale */
      [[[oTGraph oTGraphBody] cellAt: G_LEFT] setScaleIndex: 0];
      if (labeler = [oTGraph labelerForAxis: G_LEFT])
	[labeler setScaleIndex:0];
      [[[oTGraph oTGraphBody] cellAt: G_RIGHT] setScaleIndex: 0];
      if (labeler = [oTGraph labelerForAxis: G_RIGHT])
	[labeler setScaleIndex:0];
    }
  }

  /* Create main data manager */
  aDataMngr = [[OTDataMngr alloc] initFromFile: [path validString]];

  [self setOTDataMngr: aDataMngr];
  [oTGraph setOTDataMngr: aDataMngr];
  [oTLegend setOTGraph: oTGraph];
  [titleF setStringValue: [[aDataMngr dataLabel] validString]];

  [self displayAll];

  return self;
}

- setGraphX: anObject
{
  graphX = anObject;
  return self;
}

- setGraphY: anObject
{
  graphY = anObject;
  return self;
}

- setDataX: anObject
{
  dataX = anObject;
  return self;
}

- setDataY: anObject
{
  dataY = anObject;
  return self;
}

- mouseEventDidOccur: sender
{
  id cell;

  if(![[sender oTScale] validTransform])
    return nil;

  cell = [sender selectedCell];

  if([cell isKindOf: [OTAxisCell class]])
    /* AxisCell stuff */
    return [self trackAxis: sender];
  else if([cell isKindOf: [OTGraphCell class]]){ 
    /* GraphCell stuff */
    if([cell lastEventType] == NX_MOUSEUP)
      return [self attachLocation: sender];
    else
      return [self updatePosition: sender];
  }

  return self;
}


- updatePosition: sender
{
  NXPoint *location;
  id cursorImage;

  if(!graphEnabled)
    return nil;

  if(!myCursor){
    NXPoint spot;
    cursorImage = [[NXImage alloc] initFromSection:"Cross.tiff"];
    myCursor = [[NXCursor alloc] initFromImage:cursorImage];
    spot.x = 7.0; spot.y = 7.0;
    [myCursor setHotSpot:&spot];
  }

  if(crossLives == NO){
    [myCursor push];
    crossLives = YES;
  }

  location = [[sender selectedCell] lastGraphPoint];
  [graphX setDoubleValue: location->x];
  [graphY setDoubleValue: location->y];
  [[sender oTScale] convertPointFromGraph: location];
  [dataX setDoubleValue: location->x];
  [dataY setDoubleValue: location->y];

  return self;
}

- attachLocation: sender
{
  NXPoint clickedValue, nearestGraphPoint;
  id graphCell, currScale;

  if(!graphEnabled)
    return nil;

  [myCursor pop];
  crossLives = NO;

  graphCell = [sender cellAt: G_GRAPH];
  clickedValue = *([graphCell lastGraphPoint]);
  
  currScale = [[sender oTScale] attachGraph: &clickedValue to: &where];
  [dataX setDoubleValue: where.x];
  [dataY setDoubleValue: where.y];

  nearestGraphPoint = where;
  [currScale convertPointFromData: &nearestGraphPoint];

  [sender lockFocus];
  if(lit){
    [graphCell highlight: &whereFrame inView:sender lit:NO];
  }else{
    [graphCell highlight: &nullFrame inView:sender lit:NO];
  }

  whereFrame.origin.x = nearestGraphPoint.x - 7.0;
  whereFrame.origin.y = nearestGraphPoint.y - 7.0;
  whereFrame.size.width = 15.0;
  whereFrame.size.height = 15.0;

  [graphCell highlight: &whereFrame inView:sender lit:YES];
  lit = YES;
  [sender unlockFocus];
  [[sender window] flushWindow];
  
  return self;
}

- trackAxis: sender
{
  id aCell,graphCell;

  aCell = [sender selectedCell];
  graphCell = [sender cellAt:G_GRAPH];

  [sender lockFocus];

  if(lineOn){
    [graphCell highlight: &lineFrame inView:sender lit:NO];
  }

  if([aCell lastEventType] == NX_MOUSEUP){
    lineOn = NO;
  }else{

    if([aCell isHorizontal]){
      lineFrame = *[[sender oTScale] graphSpace];
      lineFrame.origin.x = ([aCell lastGraphPoint])->x;
      lineFrame.size.width = 1.0;
      [graphCell highlight: &lineFrame inView:sender lit:YES];
      [axisValueField setDoubleValue: 
       ([aCell lastDataPoint])->x];
    }else{
      lineFrame = *[[sender oTScale] graphSpace];
      lineFrame.origin.y = ([aCell lastGraphPoint])->y;
      lineFrame.size.height = 1.0;
      [graphCell highlight: &lineFrame inView:sender lit:YES];
      [axisValueField setDoubleValue: 
       ([aCell lastDataPoint])->y];
    }
    lineOn = YES;
  }

  [sender unlockFocus];
  [[sender window] flushWindow];
  return self;
}

- listRangeValues: sender
{
  char buf[100];
  char textbuf[1000];
  id axisCell,graph,data,values,text;
  int axisCellType,domainIndex,rangeNum;
  int i,len;
  double value;

  axisCellType = [sender axisCellType];
  graph = [oTGraph oTGraphBody];
  axisCell = [graph cellAt:axisCellType];
  data = [oTGraph oTDataMngr];

  if([axisCell isVertical] || ![data isRegular])
    return self;

  textbuf[0] = (char) NULL;
  domainIndex = [sender selectedCol];
  if(domainIndex >= [data domainCount])
    return self;

  rangeNum = [data count];
  for(i=0; i<rangeNum; i++){
    values = [data objectAt:i];
    len = [values count];
    if(domainIndex < len){
      value = *(double *)[values elementAt: domainIndex];
      sprintf(buf,"%20s  %8.3lf\n",
	      [[values valuesLabel] validString],
	      value);
      strcat(textbuf,buf);
    }else{
      strcat(textbuf,"N/A\n");
    }
  }

  text = [scrollView docView];
  [text setText:textbuf];

  return self;
}

- setGraphCellType: sender
{
  int tag;

  tag = [[sender selectedCell] tag];

  [[oTGraph oTGraphBody] setCell:cells[tag] at:G_GRAPH];

  [self displayAll];
  [oTLegend display];

  return self;
}

- enableAxis: sender
{
  int i,rows,cols;
  id graph,labeler,axisCell;
  BOOL didExist, willExist, changed = NO;

  graph = [oTGraph oTGraphBody];
  [axisMatrix getNumRows: &rows numCols: &cols];
  for(i=0; i<rows; i++){
    didExist = (([graph cellAt: i+1]) ? YES : NO);
    willExist = [[axisMatrix cellAt:i :0] state];
    if (didExist && !willExist) {
      changed = YES;
      [oTGraph freeCellAt: i+1];
      if ((i+1 != G_MIDDLE_HOR) && (i+1 != G_MIDDLE_VERT))
	[oTGraph freeLabelerForAxis: i+1];
    } else if (!didExist && willExist){
      changed = YES;
      /* Create a new AxisCell */
      axisCell = [oTGraph newAxis:i+1];
      [axisCell setContinuous: YES];
      
      /* Create a new Labeler */
      if ((i+1 != G_MIDDLE_HOR) && (i+1 != G_MIDDLE_VERT)) {
	labeler = [oTGraph newLabelerForAxis:i+1];
	[labeler setTarget: self];
	[labeler setAction: @selector(listRangeValues:)];
	[labeler setScaleIndex: [axisCell scaleIndex]];
      }
    }
  }

  if (changed) {
    [oTGraph setOTScale: [oTGraph oTScale] withChanged: S_BOTHSPACES];
    [self displayAll];
  }

  return self;
}

- takeWidthFrom: sender
{
  [[oTGraph oTGraphBody] setAxisMajorWidth: 
   [sender doubleValue]];

  [oTGraph setOTScale: [oTGraph oTScale] 
          withChanged: S_GRAPHSPACEONLY];
  [self displayAll];
  return self;
}

- examineSingleRange: sender
{
  int rangeIndex, len, i;
  id labeler;
  id scale, subData1, subData2, subscale1, subscale2;

  rangeIndex = [sender selectedRow];
  if(!oTDataMngr)
    return nil;

  scale = [oTGraph oTScale];

  /* Remove the current subscales if already has them */
  if ([[oTGraph oTScale] hasSubscales]) {
    [[[scale oTScale: 1] oTDataMngr] free];
    [[[scale oTScale: 2] oTDataMngr] free];
    [[oTGraph oTScale] freeSubscales];
  }

  /* Create the two subscales and data for them */
  subscale1 = [[oTGraph oTScale] createSubscale];
  subData1 = [oTDataMngr createSubData];
  subscale2 = [[oTGraph oTScale] createSubscale];
  subData2 = [oTDataMngr createSubData];

  /* Add first range to subscale1 */
  [subData1 addObject: [oTDataMngr objectAt:rangeIndex]];
  [subData1 updateLimits];
  [subscale1 setOTDataMngr:subData1];
  
  /* Add rest of ranges to subscale2 */
  len = [oTDataMngr count];
  for (i=0;i<len;i++)
    if (i != rangeIndex)
      [subData2 addObject: [oTDataMngr objectAt: i]];
  [subData2 updateLimits];
  [subscale2 setOTDataMngr: subData2];

  /* Set the left and right axis (and labelers) to use the
     subscales */
  [[oTGraph cellAt: G_LEFT] setScaleIndex: 2];
  if (labeler = [oTGraph labelerForAxis: G_LEFT])
    [labeler setScaleIndex:2];
  [[oTGraph cellAt: G_RIGHT] setScaleIndex: 1];
  if (labeler = [oTGraph labelerForAxis: G_RIGHT])
    [labeler setScaleIndex:1];

  [oTGraph setOTScale: scale withChanged: S_DATASPACEONLY];

  /* disable cell graph cell inspection */
  [graphX setStringValue:""];
  [graphY setStringValue:""];
  [dataX setStringValue:""];
  [dataY setStringValue:""];
  graphEnabled = NO;

  [self displayAll];
  
  return self;
}

- examineAllRanges: sender
{
  id labeler;
  id scale;

  scale = [oTGraph oTScale];
  if (!([scale hasSubscales]))
    return nil;

  [scale freeAllSubdata];
  [scale freeSubscales];
  
  [[oTGraph cellAt: G_LEFT] setScaleIndex: 0];
  if (labeler = [oTGraph labelerForAxis: G_LEFT])
    [labeler setScaleIndex:0];
  [[oTGraph cellAt: G_RIGHT] setScaleIndex: 0];
  if (labeler = [oTGraph labelerForAxis: G_RIGHT])
    [labeler setScaleIndex:0];

  /* enable cell graph cell inspection */
  graphEnabled = YES;

  [oTGraph setOTDataMngr: oTDataMngr];

  [self displayAll];
  return self;
}

- displayAll
{
  id graph,graphCell;

  /* clean up any highlighted points */
  if(oTGraph){
    graph = [oTGraph oTGraphBody];
    graphCell = [graph cellAt:G_GRAPH];
    if(lit){
      [graph lockFocus];
      [graphCell highlight: &whereFrame inView:graph lit:NO];
      [graph unlockFocus];
      [[graph window] flushWindow];
      lit = NO;
    }
  }    

  /* reset fields */
  [[scrollView docView] setText:""];
  [graphX setStringValue:""];
  [graphY setStringValue:""];
  [dataX setStringValue:""];
  [dataY setStringValue:""];
  [axisValueField setStringValue:""];

  /* Render */
  [oTGraph display];
  [oTLegend display];

  return self;
}

@end




