/*
  File: RealTime.m
  Authors: Matt Jacobus
  Modification History:

  Copyright (c) 1991 Objective Technologies, Inc.
  All Rights Reserved.
*/

#import <appkit/appkit.h>
#import <otikit/graphkit.h>
#import "RealTime.h"



static NXRect dataSpace = {{-1.0, 50.0}, {6.0, 50.0}};
static NXSize dataSpaceStep = {1.0, 10.0};

@implementation RealTime

+ new
{
  self = [super new];
  
  data = nil;
  dataM = nil;
  graphMngr = nil;
  points = 5;

  return self;
}

- tick:sender
{
  id labeler;

  [self addData];
  labeler = [graphMngr labelerForAxis:G_BOTTOM];
/*  [self updateLabeler];*/
  [labeler label];
  [labeler updateFrame];

  [graphMngr display];
  return self;
}


- createData
{
  int len, i;

  data = [DataMngr new];

  /* Init xValues object */
  [data setXLabels:[List new]];

  /* Init yValues objects */
  len = [dataM cellCount] - 1;
  for(i=0;i<len;i++)
    [data addObject:[Values new]];

  [data setAutoLabel:NO];
  [data setRegularData:YES];
  
  return self;
}

- addData
{
  id xLabels, yValues;
  int len, i;
  id newLabel;
  double x;

  if (!data) {
    [self createData];
    [graphMngr setData:data];
    [graphMngr setScale:[graphMngr scale] withChanged:S_GRAPHSPACEONLY];
  }

  /* Add x label to end */
  xLabels = [data xLabels];
  if ([xLabels count] == points)
    [[xLabels removeObjectAt:0] free];
  newLabel = [String new:[[dataM cellAt:0 :0] stringValue]];
  [xLabels addObject:newLabel];

  /* Add y data to end of values */
  len = [dataM cellCount];
  for(i=1;i<len;i++) {
    yValues = [data objectAt:i-1];
    if ([yValues count] == points)
      [yValues removeAt:0];
    x = [[dataM cellAt:0 :i] doubleValue];
    [yValues addElement:&x];
  }

  [data updateLimits];

  return self;
}

- updateLabeler
{
  id labeler, xLabels, wCell;
  const char *label;

  labeler = [graphMngr labelerForAxis:G_BOTTOM];
  xLabels = [data xLabels];

  /* Add label to end */
  if ([labeler cellCount] == points)
    [labeler removeColAt:0 andFree:YES];
  [labeler addCol];
  label = [[xLabels lastObject] validString];
  wCell = [labeler cellAt:0 :([labeler cellCount] - 1)];
  [wCell setStringValue: label];
  [wCell setVertAlignment: WC_TOPALIGNED];

  [labeler sizeToCells];
  
  return self;
}


- setGraphMngr:anObject
{
  id scale, graph;

  graphMngr = anObject;

  [graphMngr setAutodisplay:NO];

  /* Set up the fixed data space */
  scale = [graphMngr scale];
  [scale setAutoRange:NO];
  [scale setAutoDomain:NO];
  [scale setDataSpace:&dataSpace];
  [scale setDataSpaceStep:&dataSpaceStep];

  /* Create the AxisCells and Labelers */
  [graphMngr newAxis:G_LEFT];
  [graphMngr newAxis:G_BOTTOM];
  [graphMngr newLabelerForAxis:G_LEFT];
  [graphMngr newLabelerForAxis:G_BOTTOM];

  /* Init the bottom AxisCell */
  [self initAxisCell:G_BOTTOM];

  /* Change the default GraphCell to LineCell */
  graph = [graphMngr graph];
  [[graph setCell:[LineCell new] at:G_GRAPH] free];
  
  return self;
}


- initAxisCell:(int) cellType
{
  id aCell;

  if (aCell = [[graphMngr graph] cellAt:cellType]) {
    [aCell setAutoMajor:NO];
    [aCell setAutoMinor:NO];
    [aCell setAutoSubminor:NO];
    /* Regular data has 2 more ticks than points */
    [aCell setMajorTicks:points+2];
    [aCell setMinorTicks:0];
    [aCell setSubminorTicks:0];
  }

  return self;
}
- setData:anObject
{
  data = anObject;
  return self;
}

- setDataM:anObject
{
  dataM = anObject;
  return self;
}
- setPoints:(int) thePoints
{
  points = thePoints;
  return self;
}
@end
