/*
  File: TickShow.m
  Authors: Matt Jacobus
  Modification History:

  Copyright (c) 1991 Objective Technologies, Inc.
  All Rights Reserved.
*/

#import <appkit/appkit.h>
#import <otikit/graphkit.h>
#import "TickShow.h"
#import "Ticker.h"

typedef struct _historicalData {
  double low,high,close,volume;
} historicalData,*historicalDataPtr;

static historicalData history[] = {
  {74,    76,    75,     100000},
  {74.5,  76,    75.25,  5000},
  {74.5,  76,    75.25,  0},
  {74.5,  76,    75.25,  0},
  {75.0,  77,    76,     1000},
  {75.25, 77.5,  76.5,   2000},
  {75.0,  77.0,  75.75,  1000},
  {75.0,  77.0,  75.75,  0},
  {75.0,  77.0,  75.75,  0},
  {75.0,  77.0,  75.75,  0},
  {74.5,  75.5, 74.75,   4000},
  {75,    76,   75.5,    3000},
  {0.0,0.0,0.0,0.0}
};


@implementation TickShow

+ new
{
  self = [super new];
  oTDataMngr = nil;
  oTGraph = nil;
  points = 15;
  dataSpace.origin.x = -1;
  dataSpace.origin.y = 0;
  dataSpace.size.width = points+1;
  dataSpace.size.height = 150;
  dataSpaceStep.height = 25;
  dataSpaceStep.width = 1;

  return self;
}

- setTicker:anObject
{
  ticker = anObject;
  return self;
}

- start:sender
{
  [ticker changeTickPeriod:[periodF doubleValue]];
  return self;
}

- stop:sender
{
  [ticker stopTicks];
  return self;
}


- setOTGraph:anObject
{
  id oTScale, oTGraphBody, labeler;

  oTGraph = anObject;
  oTGraphBody = [oTGraph oTGraphBody];

  [oTGraph setAutodisplay:NO];

  /* Set up the fixed domain space */
  oTScale = [oTGraph oTScale];
  [oTScale setAutoDomain:NO];
  [oTScale setDataSpace:&dataSpace];
  [oTScale setDataSpaceStep:&dataSpaceStep];
  [oTScale updateTransforms];

  /* Set up font for two labelers */
  [oTGraph setHoldLabelerFont:YES];
  labeler = [oTGraph labelerForAxis:G_LEFT];
  [labeler setFont:[Font newFont:"Helvetica" size: 8]];
  labeler = [oTGraph labelerForAxis:G_BOTTOM];
  [labeler setFont:[Font newFont:"Helvetica" size: 8]];

  /* Init the AxisCells */
  [self initBottomAxisCell];
  [self initLeftAxisCell];

  /* Indicate that the HiLoCell2D used in RealTime */
  [[oTGraphBody cellAt:G_GRAPH] setRealTime:YES];

  /* Set up the historical data */
  [self createData];
  [oTGraph setOTDataMngr:oTDataMngr];
  
  return self;
}


- initBottomAxisCell
{
  id aCell;

  aCell = [[oTGraph oTGraphBody] cellAt:G_BOTTOM];
  [aCell setAutoMajor:NO];
  [aCell setAutoMinor:NO];
  [aCell setAutoSubminor:NO];
  /* Regular data has 2 more ticks than points */
  [aCell setMajorTicks:points+2];
  [aCell setMinorTicks:0];
  [aCell setSubminorTicks:0];

  return self;
}

- initLeftAxisCell
{
  id aCell;

  aCell = [[oTGraph oTGraphBody] cellAt:G_LEFT];
  [aCell setAutoMinor:NO];
  [aCell setAutoSubminor:NO];
  [aCell setMinorTicks:0];
  [aCell setSubminorTicks:0];

  return self;
}


- addDataHigh:(double)high low:(double)low close:(double)close
  volume:(double)volume
{
  /* Add y data to end of valkues */
  [self add:low at:0];
  [self add:high at:1];
  [self add:close at:2];
  [oTDataMngr updateLimits];

  [oTGraph setOTDataMngr:oTDataMngr];
  [oTGraph quickDisplay];

  return self;
}


- add:(double) value at:(int) index
{
  id yValues;

  yValues = [oTDataMngr objectAt:index];
  if ([yValues count] == points)
    [yValues removeAt:0];
  [yValues addElement:&value];
  return self;
}
  
- createData
{
  int i;
  historicalDataPtr dat;
  id datumObj[3];

  oTDataMngr = [[OTDataMngr alloc] init];

  /* Init yValues objects */
  for(i=0;i<3;i++){
    [oTDataMngr addObject:[[OTValues alloc] init]];
    datumObj[i] = [oTDataMngr objectAt: i];
  }

  i = 0;
  while((dat = (&history[i]))->low != 0.0){
    [datumObj[0] addElement: &dat->low];
    [datumObj[1] addElement: &dat->high];
    [datumObj[2] addElement: &dat->close];
    i++;
  }
  [oTDataMngr setAutoLabel:YES];
  [oTDataMngr setRegularData:YES];

  [oTDataMngr updateLimits];

  [ticker 
   setLow:*(double *)[datumObj[0] elementAt:[datumObj[0] count]-1]];
  [ticker
   setHigh:*(double *)[datumObj[1] elementAt:[datumObj[1] count]-1]];
  [ticker
   setClose:*(double *)[datumObj[2] elementAt:[datumObj[2] count]-1]];
  
  return self;
}

- setOTDataMngr:anObject
{
  oTDataMngr = anObject;
  return self;
}


- setPoints:(int) thePoints
{
  points = thePoints;
  return self;
}

@end
