/*
  File: Ticker.m
  Authors: Matt Jacobus
  Modification History:

  Copyright (c) 1991 Objective Technologies, Inc.
  All Rights Reserved.
*/

#import <stdlib.h>
#import <appkit/appkit.h>
#import "Ticker.h"
#import "TickShow.h"

#define T_NOTENTRY -999

@implementation Ticker

+ new
{
  self = [super new];
  entryExists = NO;
  tickProb = 0.5;
  delta = 0.01;
  volume = 2000;
  return self;
}

static double
randomZeroOne(void)
{
  return (((double) rand())/((double) RAND_MAX));
}


static void
tick(DPSTimedEntry aTeNumber, double now, void *mySelf)
{
  id aSelf;

  aSelf = (id) mySelf;
  if (randomZeroOne() < [aSelf tickProb])
    [aSelf generateData];
  [[aSelf delegate] addDataHigh:[aSelf high] 
   low:[aSelf low] close:[aSelf closeP] volume:[aSelf volume]];
}

- changeTickPeriod:(double)newPeriod
{
  if (entryExists)
    DPSRemoveTimedEntry(teNumber);
  teNumber = DPSAddTimedEntry(newPeriod, tick, (void *)self, NX_BASETHRESHOLD);
  entryExists = YES;
  return self;
}

- stopTicks
{
  if (entryExists) {
    DPSRemoveTimedEntry(teNumber);
    entryExists = NO;
  }
  return self;
}

- generateData
{
  double randomNum;
  double width;
  double placement;

  if ((randomNum = randomZeroOne()) < 0.5)
    randomNum = randomNum*(-1.0);
  close = close*(1.0 + (randomNum*delta));
  width = 0.125 * (int)((randomZeroOne() * 10) + 1);
  placement = 0.125 * (int)((randomZeroOne() * 6) + 1);
  if (width < placement)
    width = placement;
  low = close - placement;
  high = close + width - placement;
  
  volume = volume*(1.0 + (randomNum*delta));

  return self;
}

- setTickProb:(double) aTickProb
{
  tickProb = aTickProb;
  return self;
}

- setDelta:(double) aDelta
{
  delta = aDelta;
  return self;
}

- setHigh:(double)aHigh
{
  high = aHigh;
  return self;
}

- setLow:(double)aLow
{
  low = aLow;
  return self;
}

- setClose:(double)aClose
{
  close = aClose;
  return self;
}

- setVolume:(double)aVolume
{
  volume = aVolume;
  return self;
}

- setDelegate:anObject
{
  delegate = anObject;
  return self;
}

- delegate
{
  return delegate;
}

- (double) high
{
  return high;
}

- (double) low
{
  return low;
}

- (double) closeP
{
  return close;
}

- (double) volume
{
  return volume;
}

- (double) tickProb
{
  return tickProb;
}

@end
